@@ -51,6 +51,7 @@ With driver:	303826  (1.5 %)
 #include <linux/pci.h>
 #include <linux/interrupt.h>
 #include <linux/moduleparam.h>
+#include <linux/sched.h>
 #include <asm/io.h>
 #include <asm/delay.h> 
 
@@ -1462,6 +1463,7 @@ static int pciradio_initialize(struct pciradio *rad)
 	int x;
 
 	/* DAHDI stuff */
+	rad->span.owner = THIS_MODULE;
 	sprintf(rad->span.name, "PCIRADIO/%d", rad->pos);
 	sprintf(rad->span.desc, "Board %d", rad->pos + 1);
 	rad->span.deflaw = DAHDI_LAW_MULAW;
