@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2005, Adaptive Digital Technologies, Inc.
- * Copyright (c) 2005-2009, Digium Incorporated
+ * Copyright (c) 2005-2010, Digium Incorporated
  *
  * File Name: GpakCust.c
  *
@@ -32,7 +32,6 @@
 
 #include <linux/version.h>
 #include <linux/types.h>
-#include <linux/delay.h>
 #include <linux/pci.h>
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,26)
 #include <linux/semaphore.h>
@@ -54,7 +53,7 @@ static rwlock_t ifacelock;
 static struct vpmadt032 *ifaces[MAX_DSP_CORES];
 
 #define vpm_info(vpm, format, arg...)         \
-        dev_info(&voicebus_get_pci_dev(vpm->vb)->dev , format , ## arg)
+	dev_info(&vpm->vb->pdev->dev , format , ## arg)
 
 static inline struct vpmadt032 *find_iface(const unsigned short dspid)
 {
@@ -72,21 +71,12 @@ static inline struct vpmadt032 *find_iface(const unsigned short dspid)
 
 static struct vpmadt032_cmd *vpmadt032_get_free_cmd(struct vpmadt032 *vpm)
 {
-	unsigned long flags;
 	struct vpmadt032_cmd *cmd;
 	might_sleep();
-	spin_lock_irqsave(&vpm->list_lock, flags);
-	if (list_empty(&vpm->free_cmds)) {
-		spin_unlock_irqrestore(&vpm->list_lock, flags);
-		cmd = kmalloc(sizeof(struct vpmadt032_cmd), GFP_KERNEL);
-		if (unlikely(!cmd))
-			return NULL;
-		memset(cmd, 0, sizeof(*cmd));
-	} else {
-		cmd = list_entry(vpm->free_cmds.next, struct vpmadt032_cmd, node);
-		list_del_init(&cmd->node);
-		spin_unlock_irqrestore(&vpm->list_lock, flags);
-	}
+	cmd = kmalloc(sizeof(struct vpmadt032_cmd), GFP_KERNEL);
+	if (unlikely(!cmd))
+		return NULL;
+	memset(cmd, 0, sizeof(*cmd));
 	init_completion(&cmd->complete);
 	return cmd;
 }
@@ -129,24 +119,32 @@ static struct vpmadt032_cmd *vpmadt032_getreg_full_async(struct vpmadt032 *vpm,
 }
 
 /* Get the results from a previous call to vpmadt032_getreg_full_async. */
-int vpmadt032_getreg_full_return(struct vpmadt032 *vpm, int pagechange,
+static int vpmadt032_getreg_full_return(struct vpmadt032 *vpm, int pagechange,
 	u16 addr, u16 *outbuf, struct vpmadt032_cmd *cmd)
 {
 	unsigned long flags;
-	int ret = -EIO;
+	unsigned long ret;
 	BUG_ON(!cmd);
-	wait_for_completion(&cmd->complete);
+
+	/* We'll wait for 200ms */
+	ret = wait_for_completion_timeout(&cmd->complete, HZ/5);
+	if (unlikely(!ret)) {
+		spin_lock_irqsave(&vpm->list_lock, flags);
+		list_del(&cmd->node);
+		spin_unlock_irqrestore(&vpm->list_lock, flags);
+		kfree(cmd);
+		return -EIO;
+	}
+
 	if (cmd->desc & __VPM150M_FIN) {
 		*outbuf = cmd->data;
 		cmd->desc = 0;
 		ret = 0;
 	}
 
-	/* Just throw this command back on the ready list. */
-	spin_lock_irqsave(&vpm->list_lock, flags);
-	list_add_tail(&cmd->node, &vpm->free_cmds);
-	spin_unlock_irqrestore(&vpm->list_lock, flags);
-	return ret;
+	list_del(&cmd->node);
+	kfree(cmd);
+	return 0;
 }
 
 /* Read one of the registers on the VPMADT032 */
@@ -223,7 +221,7 @@ struct change_order {
 
 static struct change_order *alloc_change_order(void)
 {
-	return kzalloc(sizeof(struct change_order), GFP_KERNEL);
+	return kzalloc(sizeof(struct change_order), GFP_ATOMIC);
 }
 
 static void free_change_order(struct change_order *order)
@@ -231,6 +229,25 @@ static void free_change_order(struct change_order *order)
 	kfree(order);
 }
 
+static int vpmadt032_control(struct vpmadt032 *vpm, unsigned short int channel,
+			     GpakAlgCtrl_t control_code,
+			     GPAK_AlgControlStat_t *pstatus)
+{
+	gpakAlgControlStat_t stat;
+	int retry = 4;
+	while (retry--) {
+		stat = gpakAlgControl(vpm->dspid, channel,
+				      control_code, pstatus);
+
+		if (AcDspCommFailure == stat)
+			msleep(5);
+		else
+			break;
+	}
+
+	return stat;
+}
+
 static int vpmadt032_enable_ec(struct vpmadt032 *vpm, int channel,
 			       enum adt_companding companding)
 {
@@ -241,18 +258,18 @@ static int vpmadt032_enable_ec(struct vpmadt032 *vpm, int channel,
 	control = (ADT_COMP_ALAW == companding) ? EnableALawSwCompanding :
 						  EnableMuLawSwCompanding;
  
-	if (vpm->options.debug & DEBUG_ECHOCAN) {
+	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN) {
 		const char *law;
 		law = (control == EnableMuLawSwCompanding) ? "MuLaw" : "ALaw";
 		vpm_info(vpm, "Enabling ecan on channel: %d (%s)\n",
 			 channel, law);
 	}
-	res = gpakAlgControl(vpm->dspid, channel, control, &pstatus);
+	res = vpmadt032_control(vpm, channel, control, &pstatus);
 	if (res) {
 		vpm_info(vpm, "Unable to set SW Companding on "
 			 "channel %d (reason %d)\n", channel, res);
 	}
-	res = gpakAlgControl(vpm->dspid, channel, EnableEcanA, &pstatus);
+	res = vpmadt032_control(vpm, channel, EnableEcanA, &pstatus);
 	return res;
 }
 
@@ -261,16 +278,16 @@ static int vpmadt032_disable_ec(struct vpmadt032 *vpm, int channel)
 	int res;
 	GPAK_AlgControlStat_t pstatus;
 
-	if (vpm->options.debug & DEBUG_ECHOCAN)
+	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN)
 		vpm_info(vpm, "Disabling ecan on channel: %d\n", channel);
 
-	res = gpakAlgControl(vpm->dspid, channel, BypassSwCompanding, &pstatus);
+	res = vpmadt032_control(vpm, channel, BypassSwCompanding, &pstatus);
 	if (res) {
 		vpm_info(vpm, "Unable to disable sw companding on "
 			 "echo cancellation channel %d (reason %d)\n",
 			 channel, res);
 	}
-	res = gpakAlgControl(vpm->dspid, channel, BypassEcanA, &pstatus);
+	res = vpmadt032_control(vpm, channel, BypassEcanA, &pstatus);
 	return res;
 }
 
@@ -322,7 +339,7 @@ static void update_channel_config(struct vpmadt032 *vpm, unsigned int channel,
 	GPAK_TearDownChanStat_t tstatus;
 	GpakChannelConfig_t chanconfig;
 
-	if (vpm->options.debug & DEBUG_ECHOCAN) {
+	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN) {
 		vpm_info(vpm, "Reconfiguring chan %d for nlp %d, "
 			 "nlp_thresh %d, and max_supp %d\n", channel + 1,
 			 desired->nlp_type, desired->nlp_threshold,
@@ -341,14 +358,14 @@ static void update_channel_config(struct vpmadt032 *vpm, unsigned int channel,
 		return;
 
 	if (!desired->tap_length) {
-		res = gpakAlgControl(vpm->dspid, channel,
+		res = vpmadt032_control(vpm, channel,
 				     BypassSwCompanding, &pstatus);
 		if (res) {
 			vpm_info(vpm, "Unable to disable sw companding on "
 				 "echo cancellation channel %d (reason %d)\n",
 				 channel, res);
 		}
-		gpakAlgControl(vpm->dspid, channel, BypassEcanA, &pstatus);
+		vpmadt032_control(vpm, channel, BypassEcanA, &pstatus);
 	}
 
 	return;
@@ -419,7 +436,9 @@ static void vpmadt032_check_and_schedule_update(struct vpmadt032 *vpm,
 	queue_work(vpm->wq, &vpm->work);
 }
 int vpmadt032_echocan_create(struct vpmadt032 *vpm, int channo,
-	struct dahdi_echocanparams *ecp, struct dahdi_echocanparam *p)
+			     enum adt_companding companding,
+			     struct dahdi_echocanparams *ecp,
+			     struct dahdi_echocanparam *p)
 {
 	unsigned int ret;
 	struct change_order *order = alloc_change_order();
@@ -433,7 +452,7 @@ int vpmadt032_echocan_create(struct vpmadt032 *vpm, int channo,
 		return ret;
 	}
 
-	if (vpm->options.debug & DEBUG_ECHOCAN) {
+	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN) {
 		vpm_info(vpm, "Channel is %d length %d\n",
 			 channo, ecp->tap_length);
 	}
@@ -442,7 +461,7 @@ int vpmadt032_echocan_create(struct vpmadt032 *vpm, int channo,
 	 * module. Instead, it uses tap_length to enable or disable the echo
 	 * cancellation. */
 	order->params.tap_length = (ecp->tap_length) ? 1 : 0;
-	order->params.companding = vpm->companding;
+	order->params.companding = companding;
 	order->channel = channo;
 
 	vpmadt032_check_and_schedule_update(vpm, order);
@@ -465,7 +484,7 @@ void vpmadt032_echocan_free(struct vpmadt032 *vpm, int channo,
 	order->params.nlp_max_suppress = vpm->options.vpmnlpmaxsupp;
 	order->channel = channo;
 
-	if (vpm->options.debug & DEBUG_ECHOCAN)
+	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN)
 		vpm_info(vpm, "Channel is %d length 0\n", channo);
 
 	vpmadt032_check_and_schedule_update(vpm, order);
@@ -502,7 +521,6 @@ vpmadt032_alloc(struct vpmadt032_options *options, const char *board_name)
 	spin_lock_init(&vpm->list_lock);
 	spin_lock_init(&vpm->change_list_lock);
 	INIT_LIST_HEAD(&vpm->change_list);
-	INIT_LIST_HEAD(&vpm->free_cmds);
 	INIT_LIST_HEAD(&vpm->pending_cmds);
 	INIT_LIST_HEAD(&vpm->active_cmds);
 	sema_init(&vpm->sem, 1);
@@ -533,7 +551,7 @@ vpmadt032_alloc(struct vpmadt032_options *options, const char *board_name)
 
 	if (-1 == vpm->dspid) {
 		kfree(vpm);
-		printk(KERN_NOTICE "Unable to initialize another vpmadt032 modules\n");
+		dev_notice(&vpm->vb->pdev->dev, "Unable to initialize another vpmadt032 modules\n");
 		vpm = NULL;
 	}
 
@@ -541,6 +559,47 @@ vpmadt032_alloc(struct vpmadt032_options *options, const char *board_name)
 }
 EXPORT_SYMBOL(vpmadt032_alloc);
 
+int vpmadt032_reset(struct vpmadt032 *vpm)
+{
+	int res;
+	gpakPingDspStat_t pingstatus;
+	u16 version;
+
+	might_sleep();
+
+	set_bit(VPM150M_HPIRESET, &vpm->control);
+	msleep(2000);
+	while (test_bit(VPM150M_HPIRESET, &vpm->control))
+		msleep(1);
+
+	/* Set us up to page 0 */
+	vpmadt032_setpage(vpm, 0);
+
+	res = vpmadtreg_loadfirmware(vpm->vb);
+	if (res) {
+		dev_info(&vpm->vb->pdev->dev, "Failed to load the firmware.\n");
+		return res;
+	}
+	vpm->curpage = -1;
+
+	set_bit(VPM150M_SWRESET, &vpm->control);
+	while (test_bit(VPM150M_SWRESET, &vpm->control))
+		msleep(1);
+
+	/* Set us up to page 0 */
+	pingstatus = gpakPingDsp(vpm->dspid, &version);
+	if (!pingstatus) {
+		vpm_info(vpm, "VPM present and operational "
+			"(Firmware version %x)\n", version);
+		vpm->version = version;
+		res = 0;
+	} else {
+		res = -EIO;
+	}
+	return res;
+}
+EXPORT_SYMBOL(vpmadt032_reset);
+
 int
 vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
 {
@@ -557,8 +616,13 @@ vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
 
 	might_sleep();
 
-	if (vpm->options.debug & DEBUG_ECHOCAN)
-		printk(KERN_DEBUG "VPMADT032 Testing page access: ");
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 20)
+	INIT_WORK(&vpm->work, vpmadt032_bh, vpm);
+#else
+	INIT_WORK(&vpm->work, vpmadt032_bh);
+#endif
+	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN)
+		dev_info(&vpm->vb->pdev->dev, "VPMADT032 Testing page access: ");
 
 	for (i = 0; i < 0xf; i++) {
 		int x;
@@ -566,26 +630,26 @@ vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
 			vpmadt032_setpage(vpm, i);
 			reg = vpmadt032_getpage(vpm);
 			if (reg != i) {
-				if (vpm->options.debug & DEBUG_ECHOCAN)
-					printk(KERN_DEBUG "Failed: Sent %x != %x VPMADT032 Failed HI page test\n", i, reg);
+				if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN)
+					dev_info(&vpm->vb->pdev->dev, "Failed: Sent %x != %x VPMADT032 Failed HI page test\n", i, reg);
 				res = -ENODEV;
 				goto failed_exit;
 			}
 		}
 	}
 
-	if (vpm->options.debug & DEBUG_ECHOCAN)
-		printk(KERN_DEBUG "Passed\n");
+	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN)
+		dev_info(&vpm->vb->pdev->dev, "Passed\n");
 
 	set_bit(VPM150M_HPIRESET, &vpm->control);
-	msleep(2000);
 	while (test_bit(VPM150M_HPIRESET, &vpm->control))
 		msleep(1);
+	msleep(250);
 
 	/* Set us up to page 0 */
 	vpmadt032_setpage(vpm, 0);
-	if (vpm->options.debug & DEBUG_ECHOCAN)
-		printk(KERN_DEBUG "VPMADT032 now doing address test: ");
+	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN)
+		dev_info(&vpm->vb->pdev->dev, "VPMADT032 now doing address test: ");
 
 	for (i = 0; i < 16; i++) {
 		int x;
@@ -593,15 +657,14 @@ vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
 			vpmadt032_setreg(vpm, 0x1000, i);
 			vpmadt032_getreg(vpm, 0x1000, &reg);
 			if (reg != i) {
-				printk("VPMADT032 Failed address test\n");
-				goto failed_exit;
+				printk(KERN_CONT "VPMADT032 Failed address test\n");
+				res = -EIO;
 			}
-
 		}
 	}
 
-	if (vpm->options.debug & DEBUG_ECHOCAN)
-		printk("Passed\n");
+	if (vpm->options.debug & DEBUG_VPMADT032_ECHOCAN)
+		printk(KERN_CONT "Passed\n");
 
 	set_bit(VPM150M_HPIRESET, &vpm->control);
 	while (test_bit(VPM150M_HPIRESET, &vpm->control))
@@ -609,13 +672,12 @@ vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
 
 	res = vpmadtreg_loadfirmware(vb);
 	if (res) {
-		struct pci_dev *pdev = voicebus_get_pci_dev(vb);
-		dev_printk(KERN_INFO, &pdev->dev, "Failed to load the firmware.\n");
+		dev_info(&vb->pdev->dev, "Failed to load the firmware.\n");
 		return res;
 	}
 	vpm->curpage = -1;
 
-	dev_info(&voicebus_get_pci_dev(vb)->dev, "Booting VPMADT032\n");
+	dev_info(&vb->pdev->dev, "Booting VPMADT032\n");
 	set_bit(VPM150M_SWRESET, &vpm->control);
 	while (test_bit(VPM150M_SWRESET, &vpm->control))
 		msleep(1);
@@ -623,20 +685,14 @@ vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb)
 	pingstatus = gpakPingDsp(vpm->dspid, &vpm->version);
 
 	if (!pingstatus) {
-		if (vpm->options.debug & DEBUG_ECHOCAN)
-			printk(KERN_DEBUG "Version of DSP is %x\n", vpm->version);
+		dev_info(&vb->pdev->dev, "VPM present and operational "
+			"(Firmware version %x)\n", vpm->version);
 	} else {
-		printk(KERN_NOTICE "VPMADT032 Failed! Unable to ping the DSP (%d)!\n", pingstatus);
-		res = -1;
+		dev_notice(&vpm->vb->pdev->dev, "VPMADT032 Failed! Unable to ping the DSP (%d)!\n", pingstatus);
+		res = -EIO;
 		goto failed_exit;
 	}
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
-	INIT_WORK(&vpm->work, vpmadt032_bh, vpm);
-#else
-	INIT_WORK(&vpm->work, vpmadt032_bh);
-#endif
-
 	return 0;
 
 failed_exit:
@@ -644,6 +700,7 @@ failed_exit:
 }
 EXPORT_SYMBOL(vpmadt032_init);
 
+
 void vpmadt032_get_default_parameters(struct GpakEcanParms *p)
 {
 	memset(p, 0, sizeof(*p));
@@ -676,7 +733,9 @@ void vpmadt032_free(struct vpmadt032 *vpm)
 	struct change_order *order;
 	LIST_HEAD(local_list);
 
-	BUG_ON(!vpm);
+	if (!vpm)
+		return;
+
 	BUG_ON(!vpm->wq);
 
 	destroy_workqueue(vpm->wq);
@@ -685,7 +744,6 @@ void vpmadt032_free(struct vpmadt032 *vpm)
 	spin_lock_irqsave(&vpm->list_lock, flags);
 	list_splice(&vpm->pending_cmds, &local_list);
 	list_splice(&vpm->active_cmds, &local_list);
-	list_splice(&vpm->free_cmds, &local_list);
 	spin_unlock_irqrestore(&vpm->list_lock, flags);
 
 	while (!list_empty(&local_list)) {
@@ -743,7 +801,8 @@ void gpakReadDspMemory(
 
 	vpmadt032_io_wait(vpm);
 	if ( NumWords < VPM150M_MAX_COMMANDS ) {
-		struct vpmadt032_cmd *cmds[VPM150M_MAX_COMMANDS] = {0};
+		struct vpmadt032_cmd *cmds[VPM150M_MAX_COMMANDS];
+		memset(cmds, 0, sizeof(cmds));
 		vpmadt032_setpage(vpm, DspAddress >> 16);
 		DspAddress &= 0xffff;
 		for (i=0; i < NumWords; ++i) {
@@ -789,7 +848,7 @@ void gpakWriteDspMemory(
 	struct vpmadt032 *vpm = find_iface(DspId);
 	int i;
 
-	//printk(KERN_DEBUG "Writing %d words to memory\n", NumWords);
+	//dev_info(&vpm->vb->pdev->dev, "Writing %d words to memory\n", NumWords);
 	if (vpm) {
 		for (i = 0; i < NumWords; ++i) {
 			vpmadt032_setreg(vpm, DspAddress + i, pWordValues[i]);
@@ -797,7 +856,7 @@ void gpakWriteDspMemory(
 #if 0
 		for (i = 0; i < NumWords; i++) {
 			if (wctdm_vpmadt032_getreg(wc, DspAddress + i) != pWordValues[i]) {
-				printk(KERN_NOTICE "Error in write.  Address %x is not %x\n", DspAddress + i, pWordValues[i]);
+				dev_notice(&vpm->vb->pdev->dev, "Error in write.  Address %x is not %x\n", DspAddress + i, pWordValues[i]);
 			}
 		}
 #endif
@@ -840,11 +899,11 @@ void gpakLockAccess(unsigned short DspId)
 
 	vpm = find_iface(DspId);
 
-	if (vpm) {
-		if (down_interruptible(&vpm->sem)) {
-			return;
-		}
-	}
+	if (!vpm)
+		return;
+
+	down(&vpm->sem);
+	return;
 }
 
 
