@@ -39,11 +39,12 @@
 #include <linux/device.h>
 #include <linux/completion.h>
 #include <linux/workqueue.h>
+#include <linux/delay.h>
 
 #include "gpakenum.h"
 #include "adt_lec.h"
 
-#define DEBUG_ECHOCAN (1 << 1)
+#define DEBUG_VPMADT032_ECHOCAN (1 << 4)
 
 /* Host and DSP system dependent related definitions. */
 #define MAX_DSP_CORES		128	/* maximum number of DSP cores */
@@ -114,13 +115,10 @@ struct vpmadt032 {
 	unsigned long control;
 	unsigned char curpage;
 	unsigned short version;
-	enum adt_companding companding;
 	struct adt_lec_params curecstate[MAX_CHANNELS];
 	spinlock_t change_list_lock;
 	struct list_head change_list;
 	spinlock_t list_lock;
-	/* Commands that are ready to be used. */
-	struct list_head free_cmds;
 	/* Commands that are waiting to be processed. */
 	struct list_head pending_cmds;
 	/* Commands that are currently in progress by the VPM module */
@@ -139,11 +137,14 @@ struct dahdi_echocan_state;
 
 char vpmadt032tone_to_zaptone(GpakToneCodes_t tone);
 int vpmadt032_init(struct vpmadt032 *vpm, struct voicebus *vb);
+int vpmadt032_reset(struct vpmadt032 *vpm);
 struct vpmadt032 *vpmadt032_alloc(struct vpmadt032_options *options,
 					const char *board_name);
 void vpmadt032_free(struct vpmadt032 *vpm);
 int vpmadt032_echocan_create(struct vpmadt032 *vpm, int channo,
-	struct dahdi_echocanparams *ecp, struct dahdi_echocanparam *p);
+			     enum adt_companding companding,
+			     struct dahdi_echocanparams *ecp,
+			     struct dahdi_echocanparam *p);
 void vpmadt032_echocan_free(struct vpmadt032 *vpm, int channo,
 	struct dahdi_echocan_state *ec);
 
@@ -167,6 +168,24 @@ static inline struct vpmadt032_cmd *vpmadt032_get_ready_cmd(struct vpmadt032 *vp
 	return cmd;
 }
 
+static inline void vpmadt032_resend(struct vpmadt032 *vpm)
+{
+	unsigned long flags;
+	struct vpmadt032_cmd *cmd, *temp;
+
+	BUG_ON(!vpm);
+
+	/* By moving the commands back to the pending list, they will be
+	 * transmitted when room is available */
+	spin_lock_irqsave(&vpm->list_lock, flags);
+	list_for_each_entry_safe(cmd, temp, &vpm->active_cmds, node) {
+		cmd->desc &= ~(__VPM150M_TX);
+		list_move_tail(&cmd->node, &vpm->pending_cmds);
+	}
+	spin_unlock_irqrestore(&vpm->list_lock, flags);
+}
+
+
 int vpmadt032_module_init(void);
 
 typedef __u16 DSP_WORD;			/* 16 bit DSP word */
