@@ -20,7 +20,6 @@
  * this program for more details.
  */
 
-#include <linux/autoconf.h>
 #include <linux/init.h>
 
 #include <linux/kernel.h>	/* printk() */
@@ -130,13 +129,16 @@ struct devtype {
 static struct devtype wcb4xxp =  {"Wildcard B410P", .ports = 4, .card_type = B410P  };
 static struct devtype hfc2s =	 {"HFC-2S Junghanns.NET duoBRI PCI", .ports = 2, .card_type = DUOBRI };
 static struct devtype hfc4s =	 {"HFC-4S Junghanns.NET quadBRI PCI", .ports = 4, .card_type = QUADBRI };
-static struct devtype hfc8s =	 {"HFC-4S Junghanns.NET octoBRI PCI", .ports = 8, .card_type = OCTOBRI };
+static struct devtype hfc8s =	 {"HFC-8S Junghanns.NET octoBRI PCI", .ports = 8, .card_type = OCTOBRI };
 static struct devtype hfc2s_OV = {"OpenVox B200P", .ports = 2, .card_type = B200P_OV };
 static struct devtype hfc4s_OV = {"OpenVox B400P", .ports = 4, .card_type = B400P_OV };
 static struct devtype hfc8s_OV = {"OpenVox B800P", .ports = 8, .card_type = B800P_OV };
 static struct devtype hfc2s_BN = {"BeroNet BN2S0", .ports = 2, .card_type = BN2S0 };
 static struct devtype hfc4s_BN = {"BeroNet BN4S0", .ports = 4, .card_type = BN4S0 };
 static struct devtype hfc8s_BN = {"BeroNet BN8S0", .ports = 8, .card_type = BN8S0 };
+static struct devtype hfc4s_SW = {"Swyx 4xS0 SX2 QuadBri", .ports = 4, .card_type = BSWYX_SX2 };
+static struct devtype hfc4s_EV = {"CCD HFC-4S Eval. Board", .ports = 4,
+					.card_type = QUADBRI_EVAL };
  
 #define CARD_HAS_EC(card) ((card)->card_type == B410P)
 
@@ -156,7 +158,7 @@ static const struct dahdi_echocan_ops my_ec_ops = {
 };
 
 #if 0
-static const char *wcb4xxp_rcsdata = "$RCSfile: base.c,v $ $Revision: 7539 $";
+static const char *wcb4xxp_rcsdata = "$RCSfile: base.c,v $ $Revision: 8423 $";
 static const char *build_stamp = "" __DATE__ " " __TIME__ "";
 #endif
 
@@ -1615,6 +1617,7 @@ static int hdlc_rx_frame(struct b4xxp_span *bspan)
 	char debugbuf[256];
 	unsigned long irq_flags;
 	struct b4xxp *b4 = bspan->parent;
+	unsigned char stat;
 
 	fifo = bspan->fifos[2];
 
@@ -1642,10 +1645,14 @@ static int hdlc_rx_frame(struct b4xxp_span *bspan)
 	zleft = zlen + 1;	/* include STAT byte that the HFC injects after FCS */
 
 	do {
-		if (zleft > WCB4XXP_HDLC_BUF_LEN)
+		int truncated;
+		if (zleft > WCB4XXP_HDLC_BUF_LEN) {
+			truncated = 1;
 			j = WCB4XXP_HDLC_BUF_LEN;
-		else
+		} else {
+			truncated = 0;
 			j = zleft;
+		}
 
 		spin_lock_irqsave(&b4->fifolock, irq_flags);
 		hfc_setreg_waitbusy(b4, R_FIFO, (fifo << V_FIFO_NUM_SHIFT) | V_FIFO_DIR);
@@ -1654,8 +1661,8 @@ static int hdlc_rx_frame(struct b4xxp_span *bspan)
 		spin_unlock_irqrestore(&b4->fifolock, irq_flags);
 
 /* don't send STAT byte to DAHDI */
-		if (bspan->sigchan)
-			dahdi_hdlc_putbuf(bspan->sigchan, buf, (j == WCB4XXP_HDLC_BUF_LEN) ? j : j - 1);
+		if ((bspan->sigchan) && (j > 1))
+			dahdi_hdlc_putbuf(bspan->sigchan, buf, truncated ? j : j - 1);
 
 		zleft -= j;
 		if (DBG_HDLC && DBG_SPANFILTER) {
@@ -1664,6 +1671,7 @@ static int hdlc_rx_frame(struct b4xxp_span *bspan)
 			for (i=0; i < j; i++) printk("%02x%c", buf[i], (i < ( j - 1)) ? ' ':'\n');
 		}
 	} while (zleft > 0);
+	stat = buf[j - 1];
 
 /* Frame received, increment F2 and get an updated count of frames left */
 	spin_lock_irqsave(&b4->fifolock, irq_flags);
@@ -1682,7 +1690,6 @@ static int hdlc_rx_frame(struct b4xxp_span *bspan)
 			dev_notice(b4->dev, "odd, zlen less then 3?\n");
 		dahdi_hdlc_abort(bspan->sigchan, DAHDI_EVENT_ABORT);
 	} else {
-		unsigned char stat = buf[i - 1];
 
 /* if STAT != 0, indicates bad frame */
 		if (stat != 0x00) {
@@ -1828,13 +1835,14 @@ static void b4xxp_init_stage1(struct b4xxp *b4)
 	b4xxp_setreg8(b4, R_SCI_MSK, 0x00);	/* mask off all S/T interrupts */
 	b4xxp_setreg8(b4, R_IRQMSK_MISC, 0x00);	/* nothing else can generate an interrupt */
 
-/*
- * set up the clock controller
- * B410P has a 24.576MHz crystal, so the PCM clock is 2x the incoming clock.
- * Other cards have a 49.152Mhz crystal, so the PCM clock equals incoming clock.
- */
+	/*
+	 * set up the clock controller B410P & Cologne Eval Board have a
+	 * 24.576MHz crystal, so the PCM clock is 2x the incoming clock.
+	 * Other cards have a 49.152Mhz crystal, so the PCM clock equals
+	 * incoming clock.
+	 */
 
-	if (b4->card_type == B410P)
+	if ((b4->card_type == B410P) || (b4->card_type == QUADBRI_EVAL))
 		b4xxp_setreg8(b4, R_BRG_PCM_CFG, 0x02);
 	else
 		b4xxp_setreg8(b4, R_BRG_PCM_CFG, V_PCM_CLK);
@@ -2252,10 +2260,6 @@ static int b4xxp_open(struct dahdi_chan *chan)
 	struct b4xxp *b4 = chan->pvt;
 	struct b4xxp_span *bspan = &b4->spans[chan->span->offset];
 
-	if (!try_module_get(THIS_MODULE)) {
-		return -EBUSY;
-	}
-
 	if (DBG_FOPS && DBG_SPANFILTER)
 		dev_info(b4->dev, "open() on chan %s (%i/%i)\n", chan->name, chan->channo, chan->chanpos);
 
@@ -2269,8 +2273,6 @@ static int b4xxp_close(struct dahdi_chan *chan)
 	struct b4xxp *b4 = chan->pvt;
 	struct b4xxp_span *bspan = &b4->spans[chan->span->offset];
 
-	module_put(THIS_MODULE);
-
 	if (DBG_FOPS && DBG_SPANFILTER)
 		dev_info(b4->dev, "close() on chan %s (%i/%i)\n", chan->name, chan->channo, chan->chanpos);
 
@@ -2334,6 +2336,7 @@ static void init_spans(struct b4xxp *b4)
 		sprintf(bspan->span.location, "PCI Bus %02d Slot %02d",
 			b4->pdev->bus->number, PCI_SLOT(b4->pdev->devfn) + 1);
 
+		bspan->span.owner = THIS_MODULE;
 		bspan->span.spanconfig = b4xxp_spanconfig;
 		bspan->span.chanconfig = b4xxp_chanconfig;
 		bspan->span.startup = b4xxp_startup;
@@ -2699,7 +2702,6 @@ static int __devinit b4xx_probe(struct pci_dev *pdev, const struct pci_device_id
 	}
 
 /* card found, enabled and main struct allocated.  Fill it out. */
-	b4->magic = WCB4XXP_MAGIC;
 	b4->variety = dt->desc;
 	b4->card_type = dt->card_type;
 	b4->pdev = pdev;
@@ -2869,7 +2871,6 @@ static void __devexit b4xxp_remove(struct pci_dev *pdev)
 		pci_disable_device(pdev);
 
 		b4->ioaddr = b4->addr = NULL;
-		b4->magic = 0;
 
 		tasklet_kill(&b4->b4xxp_tlet);
 
@@ -2883,15 +2884,22 @@ static void __devexit b4xxp_remove(struct pci_dev *pdev)
 static struct pci_device_id b4xx_ids[] __devinitdata =
 {
 	{ 0xd161, 0xb410, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&wcb4xxp },
-	{ 0x1397, 0x16b8, 0x1397, 0xe552, 0, 0, (unsigned long)&hfc8s },
+	{ 0x1397, 0x16b8, 0x1397, 0xb552, 0, 0, (unsigned long)&hfc8s },
+	{ 0x1397, 0x16b8, 0x1397, 0xb55b, 0, 0, (unsigned long)&hfc8s },
 	{ 0x1397, 0x08b4, 0x1397, 0xb520, 0, 0, (unsigned long)&hfc4s },
+	{ 0x1397, 0x08b4, 0x1397, 0xb550, 0, 0, (unsigned long)&hfc4s },
 	{ 0x1397, 0x08b4, 0x1397, 0xb556, 0, 0, (unsigned long)&hfc2s },
 	{ 0x1397, 0x08b4, 0x1397, 0xe884, 0, 0, (unsigned long)&hfc2s_OV },
 	{ 0x1397, 0x08b4, 0x1397, 0xe888, 0, 0, (unsigned long)&hfc4s_OV },
 	{ 0x1397, 0x16b8, 0x1397, 0xe998, 0, 0, (unsigned long)&hfc8s_OV },
 	{ 0x1397, 0x08b4, 0x1397, 0xb566, 0, 0, (unsigned long)&hfc2s_BN },
+	{ 0x1397, 0x08b4, 0x1397, 0xb761, 0, 0, (unsigned long)&hfc2s_BN },
 	{ 0x1397, 0x08b4, 0x1397, 0xb560, 0, 0, (unsigned long)&hfc4s_BN },
+	{ 0x1397, 0x08b4, 0x1397, 0xb550, 0, 0, (unsigned long)&hfc4s_BN },
 	{ 0x1397, 0x16b8, 0x1397, 0xb562, 0, 0, (unsigned long)&hfc8s_BN },
+	{ 0x1397, 0x16b8, 0x1397, 0xb56b, 0, 0, (unsigned long)&hfc8s_BN },
+	{ 0x1397, 0x08b4, 0x1397, 0xb540, 0, 0, (unsigned long)&hfc4s_SW },
+	{ 0x1397, 0x08b4, 0x1397, 0x08b4, 0, 0, (unsigned long)&hfc4s_EV },
 	{0, }
 
 };
