@@ -33,7 +33,7 @@
 #include "dahdi_debug.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_bri.c 6659 2009-05-27 12:48:09Z tzafrir $";
+static const char rcsid[] = "$Id: card_bri.c 8545 2010-04-19 19:42:53Z tzafrir $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM(uint, poll_interval, 500, 0644, "Poll channel state interval in milliseconds (0 - disable)");
@@ -491,7 +491,8 @@ static int bri_check_stat(xpd_t *xpd, struct dahdi_chan *dchan, byte *buf, int l
 	return 0;
 }
 
-int bri_hdlc_putbuf(xpd_t *xpd, struct dahdi_chan *dchan, unsigned char *buf, int len)
+static int bri_hdlc_putbuf(xpd_t *xpd, struct dahdi_chan *dchan,
+		unsigned char *buf, int len)
 {
 #ifdef	CONFIG_DAHDI_BRI_DCHANS
 	struct BRI_priv_data	*priv;
@@ -519,7 +520,7 @@ int bri_hdlc_putbuf(xpd_t *xpd, struct dahdi_chan *dchan, unsigned char *buf, in
 	return 0;
 }
 
-void bri_hdlc_finish(xpd_t *xpd, struct dahdi_chan *dchan)
+static void bri_hdlc_finish(xpd_t *xpd, struct dahdi_chan *dchan)
 {
 	struct BRI_priv_data	*priv;
 
@@ -691,7 +692,8 @@ static void bri_hdlc_hard_xmit(struct dahdi_chan *chan)
 }
 #endif
 
-int bri_hdlc_getbuf(struct dahdi_chan *dchan, unsigned char *buf, unsigned int *size)
+static int bri_hdlc_getbuf(struct dahdi_chan *dchan, unsigned char *buf,
+		unsigned int *size)
 {
 	int			len = *size;
 	int			eoframe;
@@ -872,6 +874,7 @@ static int BRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 		/* Nothing to do yet */
 		return 0;
 	}
+	xpd->span.owner = THIS_MODULE;
 	xpd->span.spantype = "BRI";
 	xpd->span.linecompat = DAHDI_CONFIG_AMI | DAHDI_CONFIG_CCS;
 	xpd->span.deflaw = DAHDI_LAW_ALAW;
@@ -1274,11 +1277,13 @@ static int bri_shutdown(struct dahdi_span *span)
 	return 0;
 }
 
-void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t dont_care)
+static void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd,
+		xpp_line_t dont_care)
 {
 	int		i;
 	int		line_count;
 	xpp_line_t	pcm_mask;
+	uint		pcm_len;
 	xpd_t		*main_xpd;
 	unsigned long	flags;
 
@@ -1308,8 +1313,7 @@ void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t dont_care)
 			}
 			/* subunits have fake pcm_len and wanted_pcm_mask */
 			if(i > 0) {
-				sub_xpd->pcm_len = 0;
-				sub_xpd->wanted_pcm_mask = lines;
+				update_wanted_pcm_mask(sub_xpd, lines, 0);
 			}
 		}
 	}
@@ -1324,13 +1328,10 @@ void BRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd, xpp_line_t dont_care)
 	/*
 	 * The main unit account for all subunits (pcm_len and wanted_pcm_mask).
 	 */
-	main_xpd->pcm_len = (line_count)
+	pcm_len = (line_count)
 		? RPACKET_HEADERSIZE + sizeof(xpp_line_t) + line_count * DAHDI_CHUNKSIZE
 		: 0L;
-	main_xpd->wanted_pcm_mask = pcm_mask;
-	XPD_DBG(SIGNAL, main_xpd, "pcm_len=%d wanted_pcm_mask=0x%X (%s)\n",
-		main_xpd->pcm_len, main_xpd->wanted_pcm_mask,
-		xpd->xpdname);
+	update_wanted_pcm_mask(main_xpd, pcm_mask, pcm_len);
 	spin_unlock_irqrestore(&main_xpd->lock_recompute_pcm, flags);
 }
 
@@ -1718,7 +1719,7 @@ static int proc_bri_info_read(char *page, char **start, off_t off, int count, in
 					priv->state_register.bits.v_su_info0,
 					priv->state_register.bits.v_g2_g3);
 	} else
-		len += sprintf(page + len, "Unkown\n");
+		len += sprintf(page + len, "Unknown\n");
 	if(IS_NT(xpd)) {
 		len += sprintf(page + len, "T1 Timer: %d\n", priv->t1);
 	} else {
