@@ -31,7 +31,7 @@
 #include "dahdi_debug.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_fxo.c 6659 2009-05-27 12:48:09Z tzafrir $";
+static const char rcsid[] = "$Id: card_fxo.c 7883 2010-01-12 10:08:32Z tzafrir $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");
 static DEF_PARM(uint, poll_battery_interval, 500, 0644, "Poll battery interval in milliseconds (0 - disable)");
@@ -42,6 +42,8 @@ static DEF_PARM(int, ring_debounce, 50, 0644, "Number of ticks to debounce a fal
 static DEF_PARM(int, caller_id_style, 0, 0444, "Caller-Id detection style: 0 - [BELL], 1 - [ETSI_FSK], 2 - [ETSI_DTMF]");
 static DEF_PARM(int, power_denial_safezone, 650, 0644, "msec after offhook to ignore power-denial ( (0 - disable power-denial)");
 static DEF_PARM(int, power_denial_minlen, 80, 0644, "Minimal detected power-denial length (msec) (0 - disable power-denial)");
+static DEF_PARM(uint, battery_threshold, 3, 0644, "Minimum voltage that shows there is battery");
+static DEF_PARM(uint, battery_debounce, 1000, 0644, "Minimum interval (msec) for detection of battery off");
 
 enum cid_style {
 	CID_STYLE_BELL		= 0,	/* E.g: US (Bellcore) */
@@ -70,8 +72,6 @@ enum fxo_leds {
  * fluctuation during ring won't trigger false detection.
  */
 #define	POLREV_THRESHOLD	200
-#define	BAT_THRESHOLD		3
-#define	BAT_DEBOUNCE		1000	/* compensate for battery voltage fluctuation (in ticks) */
 #define	POWER_DENIAL_CURRENT	3
 #define	POWER_DENIAL_DELAY	2500	/* ticks */
 
@@ -170,7 +170,7 @@ struct FXO_priv_data {
 static const char *power2str(enum power_state pw)
 {
 	switch(pw) {
-		case POWER_UNKNOWN:	return "UNKOWN";
+		case POWER_UNKNOWN:	return "UNKNOWN";
 		case POWER_OFF:		return "OFF";
 		case POWER_ON:		return "ON";
 	}
@@ -509,6 +509,7 @@ static int FXO_card_dahdi_preregistration(xpd_t *xpd, bool on)
 	BUG_ON(!priv);
 	timer_count = xpd->timer_count;
 	XPD_DBG(GENERAL, xpd, "%s\n", (on)?"ON":"OFF");
+	xpd->span.owner = THIS_MODULE;
 	xpd->span.spantype = "FXO";
 	for_each_line(xpd, i) {
 		struct dahdi_chan	*cur_chan = XPD_CHAN(xpd, i);
@@ -901,7 +902,7 @@ static void update_battery_voltage(xpd_t *xpd, byte data_low, xportno_t portno)
 	priv->battery_voltage[portno] = volts;
 	if(xpd->ringing[portno])
 		goto ignore_reading;	/* ring voltage create false alarms */
-	if(abs(volts) < BAT_THRESHOLD) {
+	if(abs(volts) < battery_threshold) {
 		/*
 		 * Check for battery voltage fluctuations
 		 */
@@ -910,7 +911,7 @@ static void update_battery_voltage(xpd_t *xpd, byte data_low, xportno_t portno)
 
 			milliseconds = priv->nobattery_debounce[portno]++ *
 				poll_battery_interval;
-			if(milliseconds > BAT_DEBOUNCE) {
+			if(milliseconds > battery_debounce) {
 				LINE_DBG(SIGNAL, xpd, portno, "BATTERY OFF voltage=%d\n", volts);
 				priv->battery[portno] = BATTERY_OFF;
 				dahdi_report_battery(xpd, portno);
