@@ -30,7 +30,7 @@
 #include "xbus-core.h"
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: card_global.c 6659 2009-05-27 12:48:09Z tzafrir $";
+static const char rcsid[] = "$Id: card_global.c 8545 2010-04-19 19:42:53Z tzafrir $";
 
 DEF_PARM(charp,initdir, "/usr/share/dahdi", 0644, "The directory of card initialization scripts");
 
@@ -158,7 +158,7 @@ static int execute_chip_command(xpd_t *xpd, const int argc, char *argv[])
 			//XPD_DBG(REGS, xpd, "READING\n");
 			break;
 		default:
-			XPD_ERR(xpd, "Unkown operation type '%c'\n", op);
+			XPD_ERR(xpd, "Unknown operation type '%c'\n", op);
 			goto out;
 	}
 	addr_mode = argv[argno][1];
@@ -187,7 +187,7 @@ static int execute_chip_command(xpd_t *xpd, const int argc, char *argv[])
 			//XPD_DBG(REGS, xpd, "Multibyte (%c)\n", addr_mode);
 			break;
 		default:
-			XPD_ERR(xpd, "Unkown addressing type '%c'\n", addr_mode);
+			XPD_ERR(xpd, "Unknown addressing type '%c'\n", addr_mode);
 			goto out;
 	}
 	if(argv[argno][2] != '\0') {
@@ -402,7 +402,7 @@ int chip_proc_create(xbus_t *xbus, xpd_t *xpd)
 		XPD_ERR(xpd, "Failed to create proc file '%s'\n", CHIP_REGISTERS);
 		goto err;
 	}
-	SET_PROC_DIRENTRY_OWNER(priv->proc_xpd_chipregs);
+	SET_PROC_DIRENTRY_OWNER(xpd->proc_xpd_chipregs);
 	xpd->proc_xpd_chipregs->write_proc = proc_xpd_register_write;
 	xpd->proc_xpd_chipregs->read_proc = proc_xpd_register_read;
 	xpd->proc_xpd_chipregs->data = xpd;
