@@ -34,7 +34,7 @@
 #include "xpd.h"
 #include "xbus-core.h"
 
-static const char rcsid[] = "$Id: card_pri.c 7485 2009-11-04 21:45:55Z tzafrir $";
+static const char rcsid[] = "$Id: card_pri.c 8545 2010-04-19 19:42:53Z tzafrir $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM(uint, poll_interval, 500, 0644, "Poll channel state interval in milliseconds (0 - disable)");
@@ -84,6 +84,7 @@ static int pri_startup(struct dahdi_span *span);
 static int pri_shutdown(struct dahdi_span *span);
 static int pri_rbsbits(struct dahdi_chan *chan, int bits);
 static int pri_lineconfig(xpd_t *xpd, int lineconfig);
+static void send_idlebits(xpd_t *xpd, bool saveold);
 
 #define	PROC_REGISTER_FNAME	"slics"
 #ifdef	OLD_PROC
@@ -100,7 +101,7 @@ enum pri_protocol {
 static const char *pri_protocol_name(enum pri_protocol pri_protocol)
 {
 	static const char *protocol_names[] = {
-		[PRI_PROTO_0] = "??",	/* unkown */
+		[PRI_PROTO_0] = "??",	/* unknown */
 		[PRI_PROTO_E1] = "E1",
 		[PRI_PROTO_T1] = "T1",
 		[PRI_PROTO_J1] = "J1"
@@ -306,7 +307,7 @@ struct pri_leds {
 #define	VAL_PC_GPOH	0x0A	/* General Purpose Output, high level */
 #define	VAL_PC_GPOL	0x0B	/* General Purpose Output, low level */
 
-#define	NUM_CAS_RS	(REG_RS16_E - REG_RS1_E + 1)
+#define	NUM_CAS_RS_E	(REG_RS16_E - REG_RS2_E + 1)
 /* and of those, the ones used in T1: */
 #define	NUM_CAS_RS_T	(REG_RS12_E - REG_RS1_E + 1)
 
@@ -328,8 +329,8 @@ struct PRI_priv_data {
 #define	VALID_DCHAN(p)	(DCHAN(p) != NO_DCHAN)
 #define	SET_DCHAN(p,d)	do { DCHAN(p) = (d); } while(0);
 
-	byte				cas_rs_e[NUM_CAS_RS];
-	byte				cas_ts_e[NUM_CAS_RS];
+	byte				cas_rs_e[NUM_CAS_RS_E];
+	byte				cas_ts_e[NUM_CAS_RS_E];
 	uint				cas_replies;
 	bool				is_esf;
 	bool				local_loopback;
@@ -420,6 +421,78 @@ static int pri_write_reg(xpd_t *xpd, int regnum, byte val)
 			);
 }
 
+static int cas_regbase(xpd_t *xpd)
+{
+	struct PRI_priv_data	*priv;
+
+	priv = xpd->priv;
+	switch (priv->pri_protocol) {
+	case PRI_PROTO_E1:
+		return REG_RS2_E;
+	case PRI_PROTO_T1:
+		/* fall-through */
+	case PRI_PROTO_J1:
+		return REG_RS1_E;
+	case PRI_PROTO_0:
+		/* fall-through */
+		;
+	}
+	BUG();
+	return 0;
+}
+
+static int cas_numregs(xpd_t *xpd)
+{
+	struct PRI_priv_data	*priv;
+
+	priv = xpd->priv;
+	switch (priv->pri_protocol) {
+	case PRI_PROTO_E1:
+		return NUM_CAS_RS_E;
+	case PRI_PROTO_T1:
+		/* fall-through */
+	case PRI_PROTO_J1:
+		return NUM_CAS_RS_T;
+	case PRI_PROTO_0:
+		/* fall-through */
+		;
+	}
+	BUG();
+	return 0;
+}
+
+static int write_cas_reg(xpd_t *xpd, int rsnum, byte val)
+{
+	struct PRI_priv_data	*priv;
+	int			regbase = cas_regbase(xpd);
+	int			num_cas_rs = cas_numregs(xpd);
+	int			regnum;
+	bool			is_d4 = 0;
+
+	BUG_ON(!xpd);
+	priv = xpd->priv;
+	if (priv->pri_protocol && !priv->is_esf) {
+		/* same data should be copied to RS7..12 in D4 only */
+		is_d4 = 1;
+	}
+	if (rsnum < 0 || rsnum >= num_cas_rs) {
+		XPD_ERR(xpd, "RBS(TX): rsnum=%d\n", rsnum);
+		BUG();
+	}
+	regnum = regbase + rsnum;
+	priv->cas_ts_e[rsnum] = val;
+	XPD_DBG(SIGNAL, xpd, "RBS(TX): reg=0x%X val=0x%02X\n", regnum, val);
+	write_subunit(xpd, regbase + rsnum, val);
+	if (is_d4) {
+		/* same data should be copied to RS7..12 in D4 only */
+		regnum = REG_RS7_E + rsnum;
+		XPD_DBG(SIGNAL, xpd, "RBS(TX): reg=0x%X val=0x%02X\n",
+				regnum, val);
+		write_subunit(xpd, regnum, val);
+	}
+	return 0;
+}
+
 #ifdef	OLD_PROC
 static void pri_proc_remove(xbus_t *xbus, xpd_t *xpd)
 {
@@ -478,25 +551,44 @@ static bool valid_pri_modes(const xpd_t *xpd)
 	return 1;
 }
 
-static void pri_pcm_update(xpd_t *xpd)
+static void PRI_card_pcm_recompute(xbus_t *xbus, xpd_t *xpd,
+		xpp_line_t pcm_mask)
 {
 	struct PRI_priv_data	*priv;
-	int			channels = xpd->channels;
-	xpp_line_t		mask = BITMASK(xpd->channels);
-	uint			pcm_len;
+	int			i;
+	int			line_count = 0;
 	unsigned long		flags;
+	uint			pcm_len;
 
+	BUG_ON(!xpd);
 	priv = xpd->priv;
+	spin_lock_irqsave(&xpd->lock_recompute_pcm, flags);
+	//XPD_DBG(SIGNAL, xpd, "pcm_mask=0x%X\n", pcm_mask);
+	/* Add/remove all the trivial cases */
+	pcm_mask |= xpd->offhook_state;
+	for_each_line(xpd, i)
+		if (IS_SET(pcm_mask, i))
+			line_count++;
 	if(priv->is_cas) {
 		if(priv->pri_protocol == PRI_PROTO_E1) {
 			/* CAS: Don't send PCM to D-Channel */
-			channels--;
-			mask &= ~BIT(PRI_DCHAN_IDX(priv));
+			line_count--;
+			pcm_mask &= ~BIT(PRI_DCHAN_IDX(priv));
 		}
 	}
-	pcm_len = RPACKET_HEADERSIZE + sizeof(xpp_line_t)  +  channels * DAHDI_CHUNKSIZE;
-	spin_lock_irqsave(&xpd->lock_recompute_pcm, flags);
-	update_wanted_pcm_mask(xpd, mask, pcm_len);
+	/*
+	 * FIXME: Workaround a bug in sync code of the Astribank.
+	 *        Send dummy PCM for sync.
+	 */
+	if (xpd->addr.unit == 0 && pcm_mask == 0) {
+		pcm_mask = BIT(0);
+		line_count = 1;
+	}
+	pcm_len = (line_count)
+		? RPACKET_HEADERSIZE + sizeof(xpp_line_t) +
+			line_count * DAHDI_CHUNKSIZE
+		: 0L;
+	update_wanted_pcm_mask(xpd, pcm_mask, pcm_len);
 	spin_unlock_irqrestore(&xpd->lock_recompute_pcm, flags);
 }
 
@@ -558,7 +650,8 @@ static int set_pri_proto(xpd_t *xpd, enum pri_protocol set_proto)
 	priv->pri_protocol = set_proto;
 	priv->is_cas = -1;
 	xpd->channels = pri_num_channels(set_proto);
-	pri_pcm_update(xpd);
+	xpd->offhook_state = BITMASK(xpd->channels);
+	CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
 	priv->deflaw = deflaw;
 	priv->dchan_num = dchan_num;
 	priv->local_loopback = 0;
@@ -943,7 +1036,7 @@ static int pri_lineconfig(xpd_t *xpd, int lineconfig)
 		force_cas = 1;
 		set_mode_cas(xpd, 1);
 	}
-	pri_pcm_update(xpd);
+	CALL_XMETHOD(card_pcm_recompute, xpd->xbus, xpd, 0);
 	/*
 	 * E1's can enable CRC checking
 	 * CRC4 is legal only for E1, and it is checked by pri_linecompat()
@@ -1165,6 +1258,19 @@ static int PRI_card_remove(xbus_t *xbus, xpd_t *xpd)
 	return 0;
 }
 
+#ifdef	DAHDI_AUDIO_NOTIFY
+static int pri_audio_notify(struct dahdi_chan *chan, int on)
+{
+	xpd_t		*xpd = chan->pvt;
+	int		pos = chan->chanpos - 1;
+
+	BUG_ON(!xpd);
+	LINE_DBG(SIGNAL, xpd, pos, "PRI-AUDIO: %s\n", (on) ? "on" : "off");
+	mark_offhook(xpd, pos, on);
+	return 0;
+}
+#endif
+
 static int PRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 {
 	xbus_t			*xbus;
@@ -1184,6 +1290,7 @@ static int PRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 		/* Nothing to do yet */
 		return 0;
 	}
+	xpd->span.owner = THIS_MODULE;
 	xpd->span.spantype = pri_protocol_name(priv->pri_protocol);
 	xpd->span.linecompat = pri_linecompat(priv->pri_protocol);
 	xpd->span.deflaw = priv->deflaw;
@@ -1205,12 +1312,14 @@ static int PRI_card_dahdi_preregistration(xpd_t *xpd, bool on)
 	}
 	if(!priv->is_cas)
 		clear_bit(DAHDI_FLAGBIT_RBS, &xpd->span.flags);
-	xpd->offhook_state = xpd->wanted_pcm_mask;
 	xpd->span.spanconfig = pri_spanconfig;
 	xpd->span.chanconfig = pri_chanconfig;
 	xpd->span.startup = pri_startup;
 	xpd->span.shutdown = pri_shutdown;
 	xpd->span.rbsbits = pri_rbsbits;
+#ifdef	DAHDI_AUDIO_NOTIFY
+	xpd->span.audio_notify = pri_audio_notify;
+#endif
 	return 0;
 }
 
@@ -1362,18 +1471,24 @@ static int PRI_card_tick(xbus_t *xbus, xpd_t *xpd)
 
 static int PRI_card_ioctl(xpd_t *xpd, int pos, unsigned int cmd, unsigned long arg)
 {
+	struct dahdi_chan	*chan;
+
 	BUG_ON(!xpd);
 	if(!XBUS_IS(xpd->xbus, READY))
 		return -ENODEV;
+	chan = XPD_CHAN(xpd, pos);
 	switch (cmd) {
 		case DAHDI_TONEDETECT:
 			/*
 			 * Asterisk call all span types with this (FXS specific)
 			 * call. Silently ignore it.
 			 */
-			LINE_DBG(SIGNAL, xpd, pos, "PRI: Starting a call\n");
-			/* fall-through */
+			LINE_DBG(SIGNAL, xpd, pos, "PRI: TONEDETECT (%s)\n",
+				(chan->flags & DAHDI_FLAG_AUDIO) ?
+					"AUDIO" : "SILENCE");
+			return -ENOTTY;
 		case DAHDI_ONHOOKTRANSFER:
+			LINE_DBG(SIGNAL, xpd, pos, "PRI: ONHOOKTRANSFER\n");
 			return -ENOTTY;
 		default:
 			report_bad_ioctl(THIS_MODULE->name, xpd, pos, cmd);
@@ -1436,10 +1551,7 @@ static int encode_rbsbits_e1(xpd_t *xpd, int pos, int bits)
 {
 	struct PRI_priv_data	*priv;
 	byte			val;
-	int			reg_pos;
-	int			regnum;
-	unsigned long		flags;
-	
+	int			rsnum;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
@@ -1451,22 +1563,17 @@ static int encode_rbsbits_e1(xpd_t *xpd, int pos, int bits)
 		XPD_NOTICE(xpd, "%s: pos=%d out of range. Ignore\n", __FUNCTION__, pos);
 		return 0;
 	}
-	spin_lock_irqsave(&xpd->lock, flags);
 	if(pos >= 16) {
 		/* Low nibble */
-		reg_pos = pos - 16;
-		val = (priv->cas_ts_e[reg_pos] & 0xF0) | (bits & 0x0F);
+		rsnum = pos - 16;
+		val = (priv->cas_ts_e[rsnum] & 0xF0) | (bits & 0x0F);
 	} else {
 		/* High nibble */
-		reg_pos = pos;
-		val = (priv->cas_ts_e[reg_pos] & 0x0F) | ((bits << 4) & 0xF0);
+		rsnum = pos;
+		val = (priv->cas_ts_e[rsnum] & 0x0F) | ((bits << 4) & 0xF0);
 	}
-	regnum = REG_RS2_E + reg_pos;
-	priv->cas_ts_e[reg_pos] = val;
-	spin_unlock_irqrestore(&xpd->lock, flags);
-	LINE_DBG(SIGNAL, xpd, pos, "RBS: TX: bits=0x%X (reg=0x%X val=0x%02X)\n",
-		bits, regnum, val);
-	write_subunit(xpd, regnum, val);
+	LINE_DBG(SIGNAL, xpd, pos, "RBS: TX: bits=0x%X\n", bits);
+	write_cas_reg(xpd, rsnum, val);
 	return 0;
 }
 
@@ -1479,6 +1586,7 @@ static int encode_rbsbits_t1(xpd_t *xpd, int pos, int bits)
 	int			width;
 	uint			tx_bits = bits;
 	uint			mask;
+	byte			val;
 
 	BUG_ON(!xpd);
 	priv = xpd->priv;
@@ -1497,29 +1605,54 @@ static int encode_rbsbits_t1(xpd_t *xpd, int pos, int bits)
 		tx_bits >>= 2;
 	tx_bits &= BITMASK(width);
 	tx_bits <<= (chan_per_reg - offset - 1) * width;
-	if((priv->cas_ts_e[rsnum] & mask) == tx_bits) {
-#if 0
-		LINE_DBG(SIGNAL, xpd, pos, "RBS: TX: RS%02d(0x%02X, 0x%02X): REPEAT 0x%02X\n",
-			rsnum+1, mask, tx_bits, bits);
-#endif
-		return 0;
-	}
-	priv->cas_ts_e[rsnum] &= ~mask;
-	priv->cas_ts_e[rsnum] |= tx_bits;
+	val = priv->cas_ts_e[rsnum];
+	val &= ~mask;
+	val |= tx_bits;
 	LINE_DBG(SIGNAL, xpd, pos,
 		"bits=0x%02X RS%02d(%s) offset=%d tx_bits=0x%02X\n",
 		bits, rsnum+1,
 		(priv->is_esf) ? "esf" : "d4",
 		offset, tx_bits);
-	write_subunit(xpd, REG_RS1_E + rsnum , priv->cas_ts_e[rsnum]);
-	if (!priv->is_esf) {
-		/* same data should be copied to RS7..12 in D4 only */
-		write_subunit(xpd, REG_RS7_E + rsnum , priv->cas_ts_e[rsnum]);
-	}
+	write_cas_reg(xpd, rsnum , val);
 	priv->dchan_tx_counter++;
 	return 0;
 }
 
+static void send_idlebits(xpd_t *xpd, bool saveold)
+{
+	struct PRI_priv_data	*priv;
+	byte			save_rs[NUM_CAS_RS_E];
+	int			i;
+
+	if (!SPAN_REGISTERED(xpd))
+		return;
+	priv = xpd->priv;
+	BUG_ON(!priv);
+	XPD_DBG(SIGNAL, xpd, "saveold=%d\n", saveold);
+	if (saveold)
+		memcpy(save_rs, priv->cas_ts_e, sizeof(save_rs));
+	for_each_line(xpd, i) {
+		struct dahdi_chan	*chan = XPD_CHAN(xpd, i);
+
+		pri_rbsbits(chan, chan->idlebits);
+	}
+	if (saveold)
+		memcpy(priv->cas_ts_e, save_rs, sizeof(save_rs));
+}
+
+static void send_oldbits(xpd_t *xpd)
+{
+	struct PRI_priv_data	*priv;
+	int			i;
+
+	BUG_ON(!xpd);
+	priv = xpd->priv;
+	BUG_ON(!priv);
+	XPD_DBG(SIGNAL, xpd, "\n");
+	for (i = 0; i < cas_numregs(xpd); i++)
+		write_cas_reg(xpd, i , priv->cas_ts_e[i]);
+}
+
 static int pri_rbsbits(struct dahdi_chan *chan, int bits)
 {
 	xpd_t			*xpd;
@@ -1532,7 +1665,7 @@ static int pri_rbsbits(struct dahdi_chan *chan, int bits)
 	priv = xpd->priv;
 	BUG_ON(!priv);
 	if(!priv->layer1_up) {
-		XPD_DBG(SIGNAL, xpd, "RBS: TX: No layer1. Ignore.\n");
+		XPD_DBG(SIGNAL, xpd, "RBS: TX: No layer1 yet. Keep going.\n");
 	}
 	if(!priv->is_cas) {
 		XPD_NOTICE(xpd, "RBS: TX: not in CAS mode. Ignore.\n");
@@ -1633,7 +1766,7 @@ static void PRI_card_pcm_fromspan(xbus_t *xbus, xpd_t *xpd, xpacket_t *pack)
 	spin_unlock_irqrestore(&xpd->lock, flags);
 }
 
-/*! Copy PCM chunks from the packet we recieved to the xpd struct.
+/*! Copy PCM chunks from the packet we received to the xpd struct.
  * \param xbus	xbus of target xpd.
  * \param xpd	target xpd.
  * \param pack	Source packet.
@@ -1733,7 +1866,7 @@ static /* 0x33 */ HOSTCMD(PRI, SET_LED, enum pri_led_selectors led_sel, enum pri
 static void layer1_state(xpd_t *xpd, byte data_low)
 {
 	struct PRI_priv_data	*priv;
-	int			alarms = 0;
+	int			alarms = DAHDI_ALARM_NONE;
 	int			layer1_up_prev;
 
 	BUG_ON(!xpd);
@@ -1747,7 +1880,7 @@ static void layer1_state(xpd_t *xpd, byte data_low)
 	if(data_low & REG_FRS0_RRA)
 		alarms |= DAHDI_ALARM_YELLOW;
 	layer1_up_prev  = priv->layer1_up;
-	priv->layer1_up = alarms == 0;
+	priv->layer1_up = alarms == DAHDI_ALARM_NONE;
 #if 0
 	/*
 	 * Some bad bits (e.g: LMFA and NMF have no alarm "colors"
@@ -1761,13 +1894,13 @@ static void layer1_state(xpd_t *xpd, byte data_low)
 	if(!priv->layer1_up) {
 		dchan_state(xpd, 0);
 	} else if (priv->is_cas && !layer1_up_prev) {
+		int	regbase = cas_regbase(xpd);
 		int	i;
-		int	num_cas_rs = (priv->pri_protocol == PRI_PROTO_E1) ?
-				NUM_CAS_RS: NUM_CAS_RS_T;
+
 		XPD_DBG(SIGNAL , xpd,
 			"Returning From Alarm Refreshing Rx register data \n");
-		for(i = 0; i < num_cas_rs; i++)
-			query_subunit(xpd, REG_RS1_E + i);
+		for (i = 0; i < cas_numregs(xpd); i++)
+			query_subunit(xpd, regbase + i);
 	}
 
 	if(SPAN_REGISTERED(xpd) && xpd->span.alarms != alarms) {
@@ -1779,6 +1912,12 @@ static void layer1_state(xpd_t *xpd, byte data_low)
 		XPD_NOTICE(xpd, "Alarms: 0x%X (%s) => 0x%X (%s)\n",
 				xpd->span.alarms, str1,
 				alarms, str2);
+		if (priv->is_cas) {
+			if (alarms == DAHDI_ALARM_NONE)
+				send_oldbits(xpd);
+			else if (xpd->span.alarms == DAHDI_ALARM_NONE)
+				send_idlebits(xpd, 1);
+		}
 		xpd->span.alarms = alarms;
 		dahdi_alarm_notify(&xpd->span);
 		set_clocking(xpd);
@@ -1795,13 +1934,15 @@ static int decode_cas_e1(xpd_t *xpd, byte regnum, byte data_low)
 	int			rsnum = pos + 2;
 	int			chan1 = pos;
 	int			chan2 = pos + 16;
+	int			val1 = (data_low >> 4) & 0xF;
+	int			val2 = data_low & 0xF;
 
 	priv = xpd->priv;
 	BUG_ON(!priv->is_cas);
 	BUG_ON(priv->pri_protocol != PRI_PROTO_E1);
 	XPD_DBG(SIGNAL, xpd, "RBS: RX: data_low=0x%02X\n", data_low);
-	if(pos < 0 || pos >= NUM_CAS_RS) {
-		XPD_ERR(xpd, "%s: got bad pos=%d [0-%d]\n", __FUNCTION__, pos, NUM_CAS_RS);
+	if(pos < 0 || pos >= NUM_CAS_RS_E) {
+		XPD_ERR(xpd, "%s: got bad pos=%d [0-%d]\n", __FUNCTION__, pos, NUM_CAS_RS_E);
 		return -EINVAL;
 	}
 	if(chan1 < 0 || chan1 > xpd->channels) {
@@ -1818,28 +1959,14 @@ static int decode_cas_e1(xpd_t *xpd, byte regnum, byte data_low)
 			chan2);
 		return -EINVAL;
 	}
-	if(priv->cas_rs_e[pos] != data_low) {
-		int	old1 = (priv->cas_rs_e[pos] >> 4) & 0xF;
-		int	old2 = priv->cas_rs_e[pos] & 0xF;
-		int	new1 = (data_low >> 4) & 0xF;
-		int	new2 = data_low & 0xF;
-
-		XPD_DBG(SIGNAL, xpd, "RBS: RX: RS%02d (channel %2d, channel %2d): 0x%02X -> 0x%02X\n",
-				rsnum, chan1+1, chan2+1, priv->cas_rs_e[pos], data_low);
-		if(SPAN_REGISTERED(xpd)) {
-			if(old1 != new1)
-				dahdi_rbsbits(XPD_CHAN(xpd, chan1), new1);
-			if(old2 != new2)
-				dahdi_rbsbits(XPD_CHAN(xpd, chan2), new2);
-		}
-		priv->dchan_rx_counter++;
-		priv->cas_rs_e[pos] = data_low;
-	} else {
-#if 0
-		XPD_DBG(SIGNAL, xpd, "RBS: RX: RS%02d (channel %2d, channel %2d): REPEAT 0x%02X\n",
-			rsnum, chan1+1, chan2+1, priv->cas_rs_e[pos]);
-#endif
+	XPD_DBG(SIGNAL, xpd, "RBS: RX: RS%02d (channel %2d, channel %2d): 0x%02X -> 0x%02X\n",
+		rsnum, chan1+1, chan2+1, priv->cas_rs_e[pos], data_low);
+	if(SPAN_REGISTERED(xpd)) {
+		dahdi_rbsbits(XPD_CHAN(xpd, chan1), val1);
+		dahdi_rbsbits(XPD_CHAN(xpd, chan2), val2);
 	}
+	priv->dchan_rx_counter++;
+	priv->cas_rs_e[pos] = data_low;
 	return 0;
 }
 
@@ -1863,13 +1990,6 @@ static int decode_cas_t1(xpd_t *xpd, byte regnum, byte data_low)
 		rsnum = rsnum % 6;	/* 2 identical banks of 6 registers */
 	chan_per_reg = CHAN_PER_REGS(priv);
 	width = 8 / chan_per_reg;
-	if(priv->cas_rs_e[rsnum] == data_low) {
-#if 0
-		XPD_DBG(SIGNAL, xpd, "RBS: RX: RS%02d: REPEAT 0x%02X\n",
-			rsnum+1, data_low);
-#endif
-		return 0;
-	}
 	XPD_DBG(SIGNAL, xpd,
 		"RBS: RX(%s,%d): RS%02d data_low=0x%02X\n",
 		(priv->is_esf) ? "esf" : "d4",
@@ -2019,7 +2139,7 @@ static xproto_table_t PROTO_TABLE(PRI) = {
 		.card_dahdi_preregistration	= PRI_card_dahdi_preregistration,
 		.card_dahdi_postregistration	= PRI_card_dahdi_postregistration,
 		.card_tick	= PRI_card_tick,
-		.card_pcm_recompute	= generic_card_pcm_recompute,
+		.card_pcm_recompute	= PRI_card_pcm_recompute,
 		.card_pcm_fromspan	= PRI_card_pcm_fromspan,
 		.card_pcm_tospan	= PRI_card_pcm_tospan,
 		.card_ioctl	= PRI_card_ioctl,
@@ -2160,12 +2280,12 @@ static int proc_pri_info_read(char *page, char **start, off_t off, int count, in
 		len += sprintf(page + len,
 			"CAS: replies=%d\n", priv->cas_replies);
 		len += sprintf(page + len, "   CAS-TS: ");
-		for(i = 0; i < NUM_CAS_RS; i++) {
+		for(i = 0; i < NUM_CAS_RS_E; i++) {
 			len += sprintf(page + len, " %02X", priv->cas_ts_e[i]);
 		}
 		len += sprintf(page + len, "\n");
 		len += sprintf(page + len, "   CAS-RS: ");
-		for(i = 0; i < NUM_CAS_RS; i++) {
+		for(i = 0; i < NUM_CAS_RS_E; i++) {
 			len += sprintf(page + len, " %02X", priv->cas_rs_e[i]);
 		}
 		len += sprintf(page + len, "\n");
@@ -2388,12 +2508,12 @@ static DEVICE_ATTR_READER(pri_cas_show, dev, buf)
 		len += sprintf(buf + len,
 			"CAS: replies=%d\n", priv->cas_replies);
 		len += sprintf(buf + len, "   CAS-TS: ");
-		for(i = 0; i < NUM_CAS_RS; i++) {
+		for(i = 0; i < NUM_CAS_RS_E; i++) {
 			len += sprintf(buf + len, " %02X", priv->cas_ts_e[i]);
 		}
 		len += sprintf(buf + len, "\n");
 		len += sprintf(buf + len, "   CAS-RS: ");
-		for(i = 0; i < NUM_CAS_RS; i++) {
+		for(i = 0; i < NUM_CAS_RS_E; i++) {
 			len += sprintf(buf + len, " %02X", priv->cas_rs_e[i]);
 		}
 		len += sprintf(buf + len, "\n");
@@ -2555,6 +2675,11 @@ static int __init card_pri_startup(void)
 	if((ret = xpd_driver_register(&pri_driver.driver)) < 0)
 		return ret;
 	INFO("revision %s\n", XPP_VERSION);
+#ifdef	DAHDI_AUDIO_NOTIFY
+	INFO("FEATURE: WITH DAHDI_AUDIO_NOTIFY\n");
+#else
+	INFO("FEATURE: WITHOUT DAHDI_AUDIO_NOTIFY\n");
+#endif
 	xproto_register(&PROTO_TABLE(PRI));
 	return 0;
 }
