@@ -43,7 +43,7 @@
 #endif
 #include "dahdi_debug.h"
 
-static const char rcsid[] = "$Id: xbus-sysfs.c 6641 2009-05-19 16:01:27Z tzafrir $";
+static const char rcsid[] = "$Id: xbus-sysfs.c 7625 2009-11-22 21:17:16Z tzafrir $";
 
 /* Command line parameters */
 extern int debug;
@@ -118,13 +118,11 @@ static DEVICE_ATTR_READER(timing_show, dev, buf)
 	do_gettimeofday(&now);
 	xbus = dev_to_xbus(dev);
 	driftinfo = &xbus->drift;
-	len += snprintf(buf + len, PAGE_SIZE - len, "DRIFT: %-3s", sync_mode_name(xbus->sync_mode));
+	len += snprintf(buf + len, PAGE_SIZE - len, "%-3s", sync_mode_name(xbus->sync_mode));
 	if(xbus->sync_mode == SYNC_MODE_PLL) {
 		len += snprintf(buf + len, PAGE_SIZE - len,
-				" %5d: jitter %4d median %4d calc_drift %3d lost (%4d,%4d) : ",
+				" %5d: lost (%4d,%4d) : ",
 					xbus->ticker.cycle,
-					driftinfo->jitter, driftinfo->median,
-					driftinfo->calc_drift,
 					driftinfo->lost_ticks, driftinfo->lost_tick_count);
 		len += snprintf(buf + len, PAGE_SIZE - len,
 				"DRIFT %3d %ld sec ago",
@@ -212,25 +210,52 @@ static DEVICE_ATTR_READER(driftinfo_show, dev, buf)
 	xbus_t			*xbus;
 	struct xpp_drift	*di;
 	struct xpp_ticker	*ticker;
+	struct timeval		now;
 	int			len = 0;
+	int			hours;
+	int			minutes;
+	int			seconds;
+	int			speed_range;
+	int			uframes_inaccuracy;
+	int			i;
 
 	xbus = dev_to_xbus(dev);
 	di = &xbus->drift;
 	ticker = &xbus->ticker;
-#define	SHOW(ptr,item)	len += snprintf(buf + len, PAGE_SIZE - len, "%-15s: %d\n", #item, (ptr)->item)
+	/*
+	 * Calculate lost ticks time
+	 */
+	do_gettimeofday(&now);
+	seconds = now.tv_sec - di->last_lost_tick.tv.tv_sec;
+	minutes = seconds / 60;
+	seconds = seconds % 60;
+	hours = minutes / 60;
+	minutes = minutes % 60;
+#define	SHOW(ptr,item)	len += snprintf(buf + len, PAGE_SIZE - len, "%-15s: %8d\n", #item, (ptr)->item)
+	len += snprintf(buf + len, PAGE_SIZE - len, "%-15s: %8d (was %d:%02d:%02d ago)\n",
+		"lost_ticks", di->lost_ticks, hours, minutes, seconds);
+	speed_range = abs(di->max_speed - di->min_speed);
+	uframes_inaccuracy = di->sync_inaccuracy / 125;
+	len += snprintf(buf + len, PAGE_SIZE - len, "%-15s: %8d ",
+		"instability", speed_range + uframes_inaccuracy);
+	if(xbus->sync_mode == SYNC_MODE_AB) {
+		buf[len++] = '-';
+	} else {
+		for(i = 0; len < PAGE_SIZE - 1 && i < speed_range + uframes_inaccuracy; i++)
+			buf[len++] = '#';
+	}
+	buf[len++] = '\n';
+	len += snprintf(buf + len, PAGE_SIZE - len, "%-15s: %8d (uframes)\n", "inaccuracy", uframes_inaccuracy);
+	len += snprintf(buf + len, PAGE_SIZE - len, "%-15s: %8d\n", "speed_range", speed_range);
 	SHOW(xbus, sync_adjustment);
-	SHOW(di, wanted_offset);
-	SHOW(di, delta_tick);
-	SHOW(di, lost_ticks);
-	SHOW(di, kicks_up);
-	SHOW(di, kicks_down);
-	SHOW(di, delta_min);
-	SHOW(di, delta_max);
-	SHOW(di, median);
-	SHOW(di, jitter);
-	SHOW(di, calc_drift);
+	len += snprintf(buf + len, PAGE_SIZE - len, "%-15s: %8d\n", "offset (usec)", di->offset_prev);
+	SHOW(di, offset_range);
+	len += snprintf(buf + len, PAGE_SIZE - len, "%-15s: %8d\n", "best_speed", (di->max_speed + di->min_speed) / 2);
+	SHOW(di, min_speed);
+	SHOW(di, max_speed);
 	SHOW(ticker, cycle);
 	SHOW(ticker, tick_period);
+	SHOW(ticker, count);
 #undef	SHOW
 	return len;
 }
@@ -636,6 +661,22 @@ static DEVICE_ATTR_READER(offhook_show, dev, buf)
 	return len;
 }
 
+static DEVICE_ATTR_READER(timing_priority_show, dev, buf)
+{
+	xpd_t			*xpd;
+	unsigned long		flags;
+	int			len = 0;
+
+	BUG_ON(!dev);
+	xpd = dev_to_xpd(dev);
+	if(!xpd)
+		return -ENODEV;
+	spin_lock_irqsave(&xpd->lock, flags);
+	len += sprintf(buf + len, "%d\n", xpd->timing_priority);
+	spin_unlock_irqrestore(&xpd->lock, flags);
+	return len;
+}
+
 static int xpd_match(struct device *dev, struct device_driver *driver)
 {
 	struct xpd_driver	*xpd_driver;
@@ -659,6 +700,7 @@ static struct device_attribute xpd_dev_attrs[] = {
 	__ATTR(span,		S_IRUGO | S_IWUSR, span_show, span_store),
         __ATTR_RO(type),
         __ATTR_RO(offhook),
+        __ATTR_RO(timing_priority),
         __ATTR_NULL,
 };
 
@@ -707,7 +749,7 @@ int xpd_device_register(xbus_t *xbus, xpd_t *xpd)
 	dev->parent = &xbus->astribank;
 	dev_set_name(dev, "%02d:%1x:%1x", xbus->num, xpd->addr.unit, 
 			xpd->addr.subunit);
-	dev->driver_data = xpd;
+	dev_set_drvdata(dev, xpd);
 	dev->release = xpd_release;
 	ret = device_register(dev);
 	if(ret) {
@@ -728,11 +770,11 @@ void xpd_device_unregister(xpd_t *xpd)
 	BUG_ON(!xbus);
 	XPD_DBG(DEVICES, xpd, "SYSFS\n");
 	dev = &xpd->xpd_dev;
-	if(!dev->driver_data)
+	if(!dev_get_drvdata(dev))
 		return;
-	BUG_ON(dev->driver_data != xpd);
+	BUG_ON(dev_get_drvdata(dev) != xpd);
 	device_unregister(dev);
-	dev->driver_data = NULL;
+	dev_set_drvdata(dev, NULL);
 }
 
 /*--------- Sysfs Device handling ----*/
@@ -746,9 +788,9 @@ void xbus_sysfs_remove(xbus_t *xbus)
 	astribank = &xbus->astribank;
 	BUG_ON(!astribank);
 	sysfs_remove_link(&astribank->kobj, "transport");
-	if(!astribank->driver_data)
+	if(!dev_get_drvdata(astribank))
 		return;
-	BUG_ON(astribank->driver_data != xbus);
+	BUG_ON(dev_get_drvdata(astribank) != xbus);
 	device_unregister(&xbus->astribank);
 }
 
@@ -764,18 +806,18 @@ int xbus_sysfs_create(xbus_t *xbus)
 	astribank->bus = &toplevel_bus_type;
 	astribank->parent = xbus->transport.transport_device;
 	dev_set_name(astribank, "xbus-%02d", xbus->num);
-	astribank->driver_data = xbus;
+	dev_set_drvdata(astribank, xbus);
 	astribank->release = astribank_release;
 	ret = device_register(astribank);
 	if(ret) {
 		XBUS_ERR(xbus, "%s: device_register failed: %d\n", __FUNCTION__, ret);
-		astribank->driver_data = NULL;
+		dev_set_drvdata(astribank, NULL);
 		goto out;
 	}
 	ret = sysfs_create_link(&astribank->kobj, &astribank->parent->kobj, "transport");
 	if(ret < 0) {
 		XBUS_ERR(xbus, "%s: sysfs_create_link failed: %d\n", __FUNCTION__, ret);
-		astribank->driver_data = NULL;
+		dev_set_drvdata(astribank, NULL);
 		goto out;
 	}
 out:
