@@ -45,7 +45,7 @@
 #include "xpp_dahdi.h"
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: xpp_dahdi.c 7493 2009-11-05 12:06:34Z tzafrir $";
+static const char rcsid[] = "$Id: xpp_dahdi.c 7690 2009-12-14 20:41:18Z tzafrir $";
 
 #ifdef CONFIG_PROC_FS
 struct proc_dir_entry *xpp_proc_toplevel = NULL;
@@ -593,12 +593,12 @@ void xbus_request_removal(xbus_t *xbus)
 		xpd_t *xpd = xpd_of(xbus, i);
 		if(xpd) {
 			if(SPAN_REGISTERED(xpd)) {
-				int i;
+				int j;
 
 				dahdi_alarm_notify(&xpd->span);
 				XPD_DBG(DEVICES, xpd, "Queuing DAHDI_EVENT_REMOVED on all channels to ask user to release them\n");
-				for (i=0; i<xpd->span.channels; i++) {
-					dahdi_qevent_lock(XPD_CHAN(xpd, i),DAHDI_EVENT_REMOVED);
+				for (j=0; j<xpd->span.channels; j++) {
+					dahdi_qevent_lock(XPD_CHAN(xpd, j),DAHDI_EVENT_REMOVED);
 				}
 			}
 			xpd_device_unregister(xpd);
@@ -1063,6 +1063,7 @@ int dahdi_register_xpd(xpd_t *xpd)
 	const xops_t	*xops;
 
 	BUG_ON(!xpd);
+
 	xops = xpd->xops;
 	xbus = xpd->xbus;
 
@@ -1135,6 +1136,10 @@ int dahdi_register_xpd(xpd_t *xpd)
 			xbus->num, xpd->addr.unit, xpd->addr.subunit, xpd->type_name);
 	XPD_DBG(GENERAL, xpd, "Registering span '%s'\n", xpd->span.desc);
 	xpd->xops->card_dahdi_preregistration(xpd, 1);
+	if(!xpd->span.owner) {
+		XPD_ERR(xpd, "NO span.owner field -- bug in low-level driver\n");
+		WARN_ON(!xpd->span.owner);
+	}
 	if(dahdi_register(&xpd->span, prefmaster)) {
 		XPD_ERR(xpd, "Failed to dahdi_register span\n");
 		return -ENODEV;
