@@ -43,7 +43,7 @@
 #endif
 #include "parport_debug.h"
 
-static const char rcsid[] = "$Id: xpp_usb.c 6642 2009-05-19 16:09:06Z tzafrir $";
+static const char rcsid[] = "$Id: xpp_usb.c 6995 2009-08-13 20:25:16Z sruffell $";
 
 static DEF_PARM(int, debug, 0, 0644, "Print DBG statements");	/* must be before dahdi_debug.h */
 static DEF_PARM(int, usb1, 0, 0644, "Allow using USB 1.1 interfaces");
@@ -975,7 +975,7 @@ static int __init xpp_usb_init(void)
 	INFO("revision %s\n", XPP_VERSION);
 	xusb_cache = kmem_cache_create("xusb_cache",
 			sizeof(xframe_t) + XFRAME_DATASIZE,
-#if LINUX_VERSION_CODE == KERNEL_VERSION(2,6,22)
+#if (LINUX_VERSION_CODE == KERNEL_VERSION(2,6,22)) && defined(CONFIG_SLUB)
 			0, SLAB_STORE_USER,
 #else
 			0, 0,
