@@ -6,7 +6,7 @@
  * written by Jim Dixon <jim@lambdatel.com>.
  *
  * Copyright (C) 2001 Jim Dixon / Zapata Telephony.
- * Copyright (C) 2001 - 2008 Digium, Inc.
+ * Copyright (C) 2001 - 2010 Digium, Inc.
  *
  * All rights reserved.
  *
@@ -701,6 +701,18 @@ enum {
 	DAHDI_FLAGBIT_HDLC56	= 20,	/*!< Sets the given channel (if in HDLC mode) to use 56K HDLC instead of 64K  */
 };
 
+struct dahdi_count {
+	__u32 fe;		/*!< Framing error counter */
+	__u32 cv;		/*!< Coding violations counter */
+	__u32 bpv;		/*!< Bipolar Violation counter */
+	__u32 crc4;		/*!< CRC4 error counter */
+	__u32 ebit;		/*!< current E-bit error count */
+	__u32 fas;		/*!< current FAS error count */
+	__u32 be;		/*!< current bit error count */
+	__u32 prbs;		/*!< current PRBS detected pattern */
+	__u32 errsec;		/*!< errored seconds */
+};
+
 /* map flagbits to flag masks */
 #define	DAHDI_FLAG(x)	(1 << (DAHDI_FLAGBIT_ ## x))
 
@@ -740,6 +752,7 @@ enum {
 
 struct dahdi_span {
 	spinlock_t lock;
+	struct module *owner;		/*!< Which module is exporting this span. */
 	void *pvt;			/*!< Private stuff */
 	char name[40];			/*!< Span name */
 	char desc[80];			/*!< Span description */
@@ -758,10 +771,7 @@ struct dahdi_span {
 	int txlevel;			/*!< Tx level */
 	int rxlevel;			/*!< Rx level */
 	int syncsrc;			/*!< current sync src (gets copied here) */
-	unsigned int bpvcount;		/*!< BPV counter */
-	unsigned int crc4count;	        /*!< CRC4 error counter */
-	unsigned int ebitcount;		/*!< current E-bit error count */
-	unsigned int fascount;		/*!< current FAS error count */
+	struct dahdi_count count;	/*!< Performance and Error counters */
 
 	int maintstat;			/*!< Maintenance state */
 	wait_queue_head_t maintq;	/*!< Maintenance queue */
@@ -838,6 +848,11 @@ struct dahdi_span {
 	/*! Opt: Used to tell an onboard HDLC controller that there is data ready to transmit */
 	void (*hdlc_hard_xmit)(struct dahdi_chan *chan);
 
+#ifdef	DAHDI_AUDIO_NOTIFY
+	/*! Opt: audio is used, don't optimize out */
+	int (*audio_notify)(struct dahdi_chan *chan, int yes);
+#endif
+
 	/* Used by DAHDI only -- no user servicable parts inside */
 	int spanno;			/*!< Span number for DAHDI */
 	int offset;			/*!< Offset within a given card */
@@ -957,7 +972,7 @@ struct dahdi_dynamic_driver {
 	/*! Flush any pending messages */
 	int (*flush)(void);
 
-	struct dahdi_dynamic_driver *next;
+	struct list_head list;
 };
 
 /*! \brief Receive a dynamic span message */
@@ -1168,9 +1183,18 @@ static inline short dahdi_txtone_nextsample(struct dahdi_chan *ss)
 /*! Maximum audio mask */
 #define DAHDI_FORMAT_AUDIO_MASK	((1 << 16) - 1)
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,14)
-#define kzalloc(a, b) kcalloc(1, a, b)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
+#define KERN_CONT ""
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 26)
+#ifndef clamp
+#define clamp(x, low, high) min(max(low, x), high)
 #endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 25)
+
+/* Some distributions backported fatal_signal_pending so we'll use a macro to
+ * override the inline functino definition. */
+#define fatal_signal_pending(p) \
+	(signal_pending((p)) && sigismember(&(p)->pending.signal, SIGKILL))
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
 static inline void list_replace(struct list_head *old, struct list_head *new)
@@ -1180,10 +1204,80 @@ static inline void list_replace(struct list_head *old, struct list_head *new)
         new->prev = old->prev;
         new->prev->next = new;
 }
-#endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 14)
+#define kzalloc(a, b) kcalloc(1, a, b)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 12)
+#define synchronize_rcu() synchronize_kernel()
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 11)
+static inline unsigned long
+wait_for_completion_timeout(struct completion *x, unsigned long timeout)
+{
+	/* There is a race condition here.  If x->done is reset to 0
+	 * before the call to wait_for_completion after this thread wakes.
+	 */
+	timeout = wait_event_timeout(x->wait, x->done, timeout);
+	if (timeout)
+		wait_for_completion(x);
+
+	return timeout;
+}
+#endif /* 2.6.11 */
+#endif /* 2.6.12 */
+#endif /* 2.6.14 */
+#endif /* 2.6.18 */
+#endif /* 2.6.25 */
+#endif /* 2.6.26 */
+#endif /* 2.6.31 */
 
 #ifndef DMA_BIT_MASK
 #define DMA_BIT_MASK(n)	(((n) == 64) ? ~0ULL : ((1ULL<<(n))-1))
 #endif
 
+/* prink-wrapper macros */
+#define	DAHDI_PRINTK(level, category, fmt, ...)	\
+	printk(KERN_ ## level "%s%s-%s: " fmt, #level, category, \
+			THIS_MODULE->name, ## __VA_ARGS__)
+#define	span_printk(level, category, span, fmt, ...)	\
+	printk(KERN_ ## level "%s%s-%s: span-%d: " fmt, #level,	\
+		category, THIS_MODULE->name, (span)->spanno, ## __VA_ARGS__)
+#define	chan_printk(level, category, chan, fmt, ...)	\
+	printk(KERN_ ## level "%s%s-%s: %d: " fmt, #level,	\
+		category, THIS_MODULE->name, (chan)->channo, ## __VA_ARGS__)
+#define	dahdi_err(fmt, ...)	DAHDI_PRINTK(ERR, "", fmt, ## __VA_ARGS__)
+#define	span_info(span, fmt, ...)	span_printk(INFO, "", span, fmt, \
+						## __VA_ARGS__)
+#define	span_notice(span, fmt, ...)	span_printk(NOTICE, "", span, fmt, \
+						## __VA_ARGS__)
+#define	span_err(span, fmt, ...)	span_printk(ERR, "", span, fmt, \
+						## __VA_ARGS__)
+#define	chan_notice(span, fmt, ...)	chan_printk(NOTICE, "", chan, fmt, \
+						## __VA_ARGS__)
+#define	chan_err(chan, fmt, ...)	chan_printk(ERR, "", chan, fmt, \
+						## __VA_ARGS__)
+
+/* The dbg_* ones use a magical variable 'debug' and the user should be
+ * aware of that.
+*/
+#ifdef DAHDI_PRINK_MACROS_USE_debug
+#ifndef	BIT	/* added in 2.6.24 */
+#define	BIT(i)		(1UL << (i))
+#endif
+/* Standard debug bit values. Any module may define others. They must
+ * be of the form DAHDI_DBG_*
+ */
+#define	DAHDI_DBG_GENERAL	BIT(0)
+#define	DAHDI_DBG_DEVICES	BIT(7)	/* instantiation/destruction etc. */
+#define	dahdi_dbg(bits, fmt, ...)	\
+	((void)((debug & (DAHDI_DBG_ ## bits)) && DAHDI_PRINTK(DEBUG, \
+			"-" #bits, "%s: " fmt, __func__, ## __VA_ARGS__)))
+#define	span_dbg(bits, span, fmt, ...)	\
+			((void)((debug & (DAHDI_DBG_ ## bits)) && \
+				span_printk(DEBUG, "-" #bits, span, "%s: " \
+					fmt, __func__, ## __VA_ARGS__)))
+#define	chan_dbg(bits, chan, fmt, ...)	\
+			((void)((debug & (DAHDI_DBG_ ## bits)) && \
+				chan_printk(DEBUG, "-" #bits, chan, \
+					"%s: " fmt, __func__, ## __VA_ARGS__)))
+#endif /* DAHDI_PRINK_MACROS_USE_debug */
+
 #endif /* _DAHDI_KERNEL_H */
